import 'package:flutter/material.dart';

import '../provider/data.dart';
import '../utils/board_utils.dart';

class Cell {
  int value;
  bool isFixed;
  int conflictsCount = 0;

  Cell(
    @required this.value,
    @required this.isFixed,
  );

  static Color cellBorderDarkColor = Colors.black;
  static Color cellBorderLightColor = Colors.grey;
  static Color cellBorderSelectedColor = Colors.red;

  Container widget(Data myProvider, Border borders, int row, int col) {
    String imageAsset = 'assets/skins/empty.png';
    if (this.value > 0) {
      imageAsset = 'assets/skins/' + myProvider.skin + '_' + this.value.toString() + '.png';
    }

    return Container(
      decoration: BoxDecoration(
        color: this.getBackgroundColor(myProvider),
        border: borders,
      ),
      child: GestureDetector(
        child: Image(
          image: AssetImage(imageAsset),
          fit: BoxFit.fill
        ),
        onTap: () {
          if (col != null && row != null) {
            if (col != myProvider.currentCellCol || row != myProvider.currentCellRow) {
              myProvider.selectCell(col, row);
            } else {
              myProvider.selectCell(null, null);
            }
          }
        },
      )
    );
  }

  Color getBackgroundColor(Data myProvider) {
    Color editableCellColor = Colors.grey[100];
    Color editableCellColorConflict = Colors.pink[100];
    Color fixedCellColor = Colors.grey[300];
    Color fixedCellColorConflict = Colors.pink[200];
    Color editableSelectedValueColor = Colors.green[100];
    Color fixedSelectedValueColor = Colors.green[300];

    Color backgroundColor = editableCellColor;

    if (this.isFixed) {
      backgroundColor = fixedCellColor;
    }

    if (myProvider.showConflicts && (this.conflictsCount != 0)) {
      if (this.isFixed) {
        backgroundColor = fixedCellColorConflict;
      } else {
        backgroundColor = editableCellColorConflict;
      }
    }

    if (myProvider.showConflicts && (this.value == myProvider.currentCellValue)) {
      if (this.isFixed) {
        backgroundColor = fixedSelectedValueColor;
      } else {
        backgroundColor = editableSelectedValueColor;
      }
    }

    return backgroundColor;
  }

  Container widgetUpdateValue(Data myProvider) {
    if (this.value < 0) {
      return Container();
    }

    String imageAsset = 'assets/skins/empty.png';
    if (this.value > 0) {
      imageAsset = 'assets/skins/' + myProvider.skin + '_' + this.value.toString() + '.png';
    }

    Color backgroundColor = Colors.grey[200];

    if (
      myProvider.showConflicts
      && myProvider.currentCellCol != null
      && myProvider.currentCellRow != null
    ) {
      if (!BoardUtils.isValueAllowed(myProvider, myProvider.currentCellCol, myProvider.currentCellRow, this.value)) {
        backgroundColor = Colors.pink[100];
      }
    }

    return Container(
      decoration: BoxDecoration(
        color: backgroundColor,
        border: Border.all(
          color: Colors.black,
          width: 2,
        ),
      ),
      child: GestureDetector(
        child: Image(
          image: AssetImage(imageAsset),
          fit: BoxFit.fill
        ),
        onTap: () {
          if (myProvider.currentCellCol != null && myProvider.currentCellRow != null) {
            myProvider.updateCellValue(myProvider.currentCellCol, myProvider.currentCellRow, this.value);
          }
          myProvider.selectCell(null, null);
          BoardUtils.computeConflictsInBoard(myProvider);
        },
      )
    );
  }

  static Border getCellBorders(Data myProvider, int row, int col) {
    int blockSizeHorizontal = myProvider.blockSizeHorizontal;
    int blockSizeVertical = myProvider.blockSizeVertical;

    double cellBorderWidth = 4;

    if (blockSizeVertical * blockSizeHorizontal > 8) {
        cellBorderWidth = 2;
        if (blockSizeVertical * blockSizeHorizontal > 10) {
            cellBorderWidth = 1;
        }
    }

    Border borders = Border.all(
      color: cellBorderSelectedColor,
      width: cellBorderWidth,
    );

    if (col != myProvider.currentCellCol || row != myProvider.currentCellRow) {
      borders = Border(
        top: BorderSide(width: cellBorderWidth, color: ((row % blockSizeVertical) == 0) ? cellBorderDarkColor : cellBorderLightColor),
        left: BorderSide(width: cellBorderWidth, color: ((col % blockSizeHorizontal) == 0) ? cellBorderDarkColor : cellBorderLightColor),
        right: BorderSide(width: cellBorderWidth, color: (((col + 1) % blockSizeHorizontal) == 0) ? cellBorderDarkColor : cellBorderLightColor),
        bottom: BorderSide(width: cellBorderWidth, color: (((row + 1) % blockSizeVertical) == 0) ? cellBorderDarkColor : cellBorderLightColor),
      );
    }

    return borders;
  }

}
